////////////
//This code generates four patterns of 16 beats each.
//LEDs blink to indicate the patterns of voltage.
//1/8-inch jacks allow the signals to be used as gates or triggers
//in an extended modular system.
//
//Knob 1 controls the master clock
//Knob 2 picks from the preset patterns
//Knob 3 sets the length of the triggers/gates - turning the knob all the way up creates ties
//
//The code is written so that each gate channel’s pattern number and trigger length can be
//changed independently. Additional knobs would provide more variety and performance control.
//For example, instead of one knob selecting the pattern number for all four channels, each of four 
// knobs can select the pattern number of a single channel, allowing for mixing patterns
//////////////


//The following variable MUST be defined as a const because it is used as the size of an array.
//using a dynamic variable in the array size is not possible
//this type of variable is often written in all caps
const byte PATTERN_COUNT = 5; //if you create more patterns, change this number

//this variable is an unsigned long because it is going to keep track of milliseconds
//it will get very large while the program is running.
unsigned long counter = 0;  //variable to keep track of the time
int beat = 0; //index variable to keep track of the current step in the 16-beat pattern

const int out1 = 2; //variables for pin connections for the gate outputs
const int out2 = 3;
const int out3 = 4;
const int out4 = 5;

const int clockKnob = A0; //variables for pin connections to knobs
const int patternKnob = A1;
const int lengthKnob = A2;

int stepTime;  //variables to keep track of the mapped knob values
int patternNum;
int trigLength;

//arrays to keep track of 16-beat patterns per gate channel
//1 means set a channel HIGH, 0 means set a channel LOW
bool gateOnePatterns[][16] = {
  {1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0},
  {1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0},
  {0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1},
  {1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0},
  {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}
};

bool gateTwoPatterns[][16] = {
  {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0},
  {0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 1, 0},
  {1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 1, 1},
  {0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1},
  {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}
};

bool gateThreePatterns[][16] = {
  {0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0},
  {1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0},
  {1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0},
  {1, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1},
  {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}
};

bool gateFourPatterns[][16] = {
  {0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0},
  {1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1},
  {1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0},
  {0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1},
  {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}
};


void setup() {
  pinMode(out1, OUTPUT);  //set gate pins as outputs
  pinMode(out2, OUTPUT);
  pinMode(out3, OUTPUT);
  pinMode(out4, OUTPUT);
}

void loop() {
  //Read Knobs and store raw values
  int clockRaw = analogRead(clockKnob);
  int patternRaw = analogRead(patternKnob);
  int lengthRaw = analogRead(lengthKnob);

  //Scale Raw values to desired ranges using the map() function
  stepTime = map(clockRaw, 0, 1023, 1000, 30); //set a clock from 1000ms between beats to 30ms (make the 1000 and 30 smaller for oscillator-like effects)
  patternNum = map(patternRaw, 0, 1023, 0, PATTERN_COUNT - 1); //select from patterns numbered 0 - 4
  trigLength = map(lengthRaw, 0, 1023, 5, stepTime); //select trig lengths from 5ms to a full beat length

  //keep track of time with custom function: stepper()
  stepper(stepTime);

  //write outputs high and low with custom function: trigger()
  trigger(1, patternNum, trigLength);
  trigger(2, patternNum, trigLength);
  trigger(3, patternNum, trigLength);
  trigger(4, patternNum, trigLength);

}

  //master timer function keeps track of which beat to play and keeps a running timer
  void stepper(unsigned long stepDur) {
    static unsigned long previousTime = 0;
    counter = millis() - previousTime;  //counter counts up from 0 to whatever the step time has been set to
  
    if (counter >= stepDur) { //if the counter reaches the stepTime, do the following:
      beat++;             //increment the beat by one
      previousTime = millis(); //update previousTime to right now
      if (beat > 15) {      //if the beat reaches 16, start over at 0
        beat = 0;
      }
    }
  }

//Custom Function to produce outputs of different lengths
//The switch...case structure looks for which Output number ("out")
//then assigns a pattern number(patternNum) from a 2D array
//then keeps track of how long to keep the output HIGH by comparing the trigLength ("dur") to the
//current time of the counter - which is updated each time through the loop by the "stepper()" function
void trigger(int out, int patternNum, int dur) {
  switch (out) {
    case 1: //if the first argument ("out") is "1", do the following:
      //check to see if the counter is less than the note length
      //AND (&&) that the pattern indicates a "1". If so, write the output HIGH to 5V
      if (counter < dur && gateOnePatterns[patternNum][beat] == 1) {
        digitalWrite(out1, HIGH);
      }

      else if (gateOnePatterns[patternNum][beat] == 0) {
        digitalWrite(out1, LOW);
      }

      //if the above cases are not true, and the trigLength is equal to the step length,
      //don't turn the gate off at the end of the step. This will create a tie between adjacent "1"s
      //in a pattern when the length knob is turned all the way to maximum
      else if (stepTime == trigLength) {
        //if the gate duration is all the way up, don't toggle the gate LOW
      }

      //in any other case, write the gate LOW once the counter passes dur
      else {
        digitalWrite(out1, LOW);
      }
      break;

    case 2:
      if (counter < dur && gateTwoPatterns[patternNum][beat] == 1) {
        digitalWrite(out2, HIGH);
      }

      else if (gateTwoPatterns[patternNum][beat] == 0) {
        digitalWrite(out2, LOW);
      }

      else if (stepTime == trigLength) {
        //if the gate duration is all the way up, don't toggle the gate LOW
      }
      else {
        digitalWrite(out2, LOW);
      }
      break;

    case 3:
      if (counter < dur && gateThreePatterns[patternNum][beat] == 1) {
        digitalWrite(out3, HIGH);
      }

      else if (gateThreePatterns[patternNum][beat] == 0) {
        digitalWrite(out3, LOW);
      }

      else if (stepTime == trigLength) {
        //if the gate duration is all the way up, don't toggle the gate LOW
      }
      else {
        digitalWrite(out3, LOW);
      }
      break;

    case 4:
      if (counter < dur && gateFourPatterns[patternNum][beat] == 1) {
        digitalWrite(out4, HIGH);
      }

      else if (gateFourPatterns[patternNum][beat] == 0) {
        digitalWrite(out4, LOW);
      }

      else if (stepTime == trigLength) {
        //if the gate duration is all the way up, don't toggle the gate LOW
      }
      else {
        digitalWrite(out4, LOW);
      }
      break;

    default:
      break;
  }
}

//**********END OF CODE
