/////////////////////////////////
//This code shows how to use millis() to keep track of the time that has elapsed since the program started.
//By comparing the current time to some known time in the past, we can keep a counter running to time our steps.
//This lets us run a sequencer without using delay() enabling multiple things to happen simultaneously.
//The sequencer turns on step[] pins in sequence. LEDs light up each step and show the pattern of the steps.
//These pins also each go through a potentiometer that can set a unique voltage for each of the 8 steps.
//These potentiometers are tied together through diodes so that steps do not affect each other.
//The voltage of the current step should be sent to the v/oct input of a VCO to control the frequency of a tone.
//In the next example, we will tie the output to an Analog Input pin of the Arduino to
//control the pitch of a built-in tone as well.
//////////////////////////////////

//bool (boolean) data type stores a variable that only has two states: true or false
bool stepStop = false;   //Is it time to STOP the current step?
bool stepStart = false; //Is it time to START the next step?

//Set up an ARRAY to represent the step pins of the sequencer:
//every time the program sees the word "steps[n]" it will
//replace it with the "n" number in the list.
//e.g. from the array below: steps[0] = 9, steps[1] = 8, steps[6] = 3, etc
int steps[8] = {9, 8, 7, 6, 5, 4, 3, 2};  //the numbers represent the Arduino Digital Pins
int stepNum = 0; //variable to use as index for "steps[ ]"

void setup() {
  pinMode(steps[0], OUTPUT); //Configure pin 9 as an output
  pinMode(steps[1], OUTPUT); //Configure pin 8 as an output
  pinMode(steps[2], OUTPUT); //Configure pin 7 as an output
  pinMode(steps[3], OUTPUT); //...
  pinMode(steps[4], OUTPUT);
  pinMode(steps[5], OUTPUT);
  pinMode(steps[6], OUTPUT);
  pinMode(steps[7], OUTPUT);
}

void loop() {
  //stepController is a custom function that updates the variables startStep and stopStep
  //the two arguments represent values for: 1. the speed of the steps, 2. the legth of the notes.
  //knobs are read into the funtion where they are scaled and used to control the sequencer
  //More details are in the body of the function declaration below (after the loop())
  stepController(1000, 500);

  //Check to see if the stepController reports that it is time to STOP the current note
  if (stepStop == true) {
    digitalWrite(steps[stepNum], LOW); //turn off the last step
    stepStop = false; //reset the flag
  }

  //Check to see if the stepController reports that it is time to START the new note
  if (stepStart == true) {
    stepNum++; //add one to stepNum. "stepNum = stepNum + 1;" would do the same thing. Odd-only counting with stepNum +2;
    if (stepNum > 7) {
      stepNum = 0;
    }
    digitalWrite(steps[stepNum], HIGH); //turn on the next step
    stepStart = false; //set our running state to indicate that we are not ready to start a new step
  }

}

//CUSTOM STEPPING FUNCTION
void stepController(int speedIn, int durIn) {

  //Variables for Keeping Track of Time (unsigned long will allow us to store very large numbers)
  static unsigned long previousTime = 0; //keep track of the time we last stepped
  static unsigned long counter = 0;  //keep track of time since the last step
  counter = millis() - previousTime;    //counter keeps track of the number of ms since last step began
  int stepInterval = speedIn;   //speedIn is a variable used to pass numbers into this function from the main loop
  int stepDuration = durIn;   //this is the gate duration. It should be less than the step interval

  if (counter > stepDuration) { //check to see if we should STOP the previous step
    stepStop = true;    //set flag to trigger the stop of the current step
  }

  if (counter >= stepInterval && stepStart == false) { //check to see if we should START the next step
    stepStart = true;   //set flag to trigger the start of the next step
    previousTime = millis(); //set new previous time to restart counter at beginning of each step
  }

  else {
    stepStart = false;    //make it unambiguous that we should NOT start a new step if the above is not true
  }

}
