/////////////////////////////////
//This code creates an 8-step CV sequencer with and internal oscillator, speed control, 
//and a Gate Output
//
//!!! add Gate Output and make steps stay on full duration for CV output
//turning the Gate Off instead of turning the step off will allow an externally connected
//Oscillator to function correctly. Otherwise, it will be given two pitches for each step - the 
//pitch selected by the knob when the step is high, and a pitch for zero volts when the step is low
//////////////////////////////////

//bool (boolean) data type stores a variable that only has two states: true or false
bool stepStop = false;   //Is it time to STOP the current step?
bool stepStart = false; //Is it time to START the next step?

//Set up an ARRAY to represent the step pins of the sequencer:
//every time the program sees the word "steps[n]" it will
//replace it with the "n" number in the list.
//e.g. from the array below: steps[0] = 9, steps[1] = 8, steps[6] = 3, etc
int steps[8] = {9, 8, 7, 6, 5, 4, 3, 2};  //the numbers represent the Arduino Digital Pins
int stepNum = 0; //variable to use as index for "steps[ ]"
int prevStepNum = 0; //!!!variable to store the previous step number


//variables related to the tones
const int tonePin = 13;   //Output pin to produce tone
const int CVPin = A5;     //Analog Input Pin to read CV
int CVVal;      //Variable to store the current level of the CV Input (read on A5)
int note;           //frequency for the tone() function to play

//Variables for keeping track of stepping speed
const int speedKnobPin = A2;

const int gatePin = 0;    //!!!pin to produce a digital gate out

void setup() {
  pinMode(steps[0], OUTPUT); //Configure pin 9 as an output
  pinMode(steps[1], OUTPUT); //Configure pin 8 as an output
  pinMode(steps[2], OUTPUT); //Configure pin 7 as an output
  pinMode(steps[3], OUTPUT);
  pinMode(steps[4], OUTPUT);
  pinMode(steps[5], OUTPUT);
  pinMode(steps[6], OUTPUT);
  pinMode(steps[7], OUTPUT);

  pinMode(gatePin, OUTPUT);

}

void loop() {
  //stepController is a custom function that steps the variables startStep and stopStep
  //the two numbers represent 1. the speed of the steps and 2. the length of the notes.
  //knobs are read into the function where they are scaled and used to control the sequencer
  //More details are in the body of the function declaration below (after the loop())
  stepController(speedKnobPin, speedKnobPin);

  //Check to see if the stepController reports that it is time to STOP the current note
  if (stepStop == true) {
    //digitalWrite(steps[stepNum], LOW);  //!!! This moves to right before a new step
    noTone(tonePin);                    //stop tone
    digitalWrite(gatePin, LOW);         //!!!toggle GATE LOW instead of turning off step
    stepStop = false;                   //reset the flag
  }

  //Check to see if the stepController reports that it is time to START the new note
  if (stepStart == true) {
    stepNum++; //add one to stepNum. "stepNum = stepNum + 1;" would do the same thing. Odd-only counting with stepNum +2;
    if (stepNum > 7) {
      stepNum = 0;
    }
      digitalWrite(steps[prevStepNum], LOW); //!!!turn off the last step

      digitalWrite(steps[stepNum], HIGH); //!!!turn on the next step
      prevStepNum = stepNum;   //!!! save this step so we can turn it off before the next new one.
      stepStart = false; //set our running state to indicate that we are not ready to start a new step


    //get value of CV from pin A5
    //Note: the (void) and the delay are there to deal with ghost voltages
    //resulting from multiplexing of the ADC and the unbuffered circuitry.
    //changes to the hardware can make this unnecessary
    (void)analogRead(CVPin);
    delay(8);
    CVVal = analogRead(CVPin);
    note = map(CVVal, 0, 1000, 31, 4878);
    tone(tonePin, note);  //write the note to the audio out
    digitalWrite(gatePin, HIGH);  //!!!turn the gate signal to +5V
  }

}

//CUSTOM STEPPING FUNCTION
void stepController(int speedIn, int durIn) {

  //Variables for Keeping Track of Time (unsigned long will allow us to store very large numbers)
  static unsigned long previousTime = 0; //keep track of the time we last stepped
  static unsigned long counter = 0;  //keep track of time since the last step
  counter = millis() - previousTime;    //counter keeps track of the number of ms since last step began
  int speedValRaw = analogRead(speedIn); //read the speed knob and store the raw value
  int stepInterval = map(speedValRaw, 0, 1023, 300, 10); //map the value to new speeds (chosen by ear)
  int durValRaw = analogRead(durIn);  //read the duration knob and store the raw value
  int stepDuration = map(durValRaw, 0, 1023, 0, stepInterval - 1); //map the duration knob to a new range from very short to an entire step - 1ms

  if (counter > stepDuration) { //check to see if we should STOP the previous step
    stepStop = true;    //set flag to trigger the stop of the current step
  }

  //Add an if statment to check for the knob to be at Zero. If so, stop the sequencer
  if (speedValRaw != 0) { //if knob is not set to 0, it sets the speed of the sequencer
    if (counter >= stepInterval && stepStart == false) { //check to see if we should START the next step
      stepStart = true;   //set flag to trigger the start of the next step
      previousTime = millis(); //set new previous time to restart counter at beginning of each step
    }

    else {
      stepStart = false;    //make it unambiguous that we should NOT start a new step
    }
  }
  else { //if knob is at zero, stop sequencer
    stepStart = false;
    stepStop = false;
  }

}
